var PWA = (function (exports) {
  function generateIcons(path = "/favicon.png") {
    return [{
      src: path,
      type: "image/png",
      sizes: "192x192"
    }, {
      src: path,
      type: "image/png",
      sizes: "512x512"
    }];
  }

  function manifest({
    name,
    icons = generateIcons(),
    display = "standalone",
    backgroundColor = "#000000",
    themeColor = "#000000",
    startUrl = "/?source=pwa"
  }) {
    return {
      name,
      short_name: name,
      icons,
      display,
      background_color: backgroundColor,
      theme_color: themeColor,
      start_url: startUrl
    };
  }

  var factory = (h => ({
    name,
    title = name,
    styles = [],
    scripts = [],
    content: Content = '',
    rootAttributes = {
      id: 'root'
    },
    lang = 'en',
    themeColor = '#000000',
    manifest = '/manifest.json',
    favicon = '/favicon.png'
  }) => h("html", {
    lang: lang
  }, h("head", null, h("title", null, title), h("meta", {
    charSet: "utf-8"
  }), h("meta", {
    name: "mobile-web-app-capable",
    content: "yes"
  }), h("meta", {
    name: "viewport",
    content: "width=device-width, initial-scale=1.0"
  }), h("meta", {
    name: "theme-color",
    content: themeColor
  }), h("meta", {
    name: "apple-mobile-web-app-capable",
    content: "yes"
  }), h("meta", {
    name: "apple-mobile-web-app-status-bar-style",
    content: "black"
  }), h("link", {
    rel: "manifest",
    href: manifest
  }), h("link", {
    rel: "shortcut icon",
    href: favicon
  }), styles.map(href => h("link", {
    key: href,
    rel: "stylesheet",
    href: href
  }))), h("body", null, h("div", rootAttributes, typeof Content === 'string' ? Content : h(Content, null)), scripts.map(src => h("script", {
    key: src,
    src: src
  })))));

  function getReact() {
    try {
      return {
        createElement: require("react").createElement,
        renderToString: require("react-dom/server").renderToString
      };
    } catch (e) {}

    return {
      createElement: require("preact").h,
      renderToString: require("preact-render-to-string")
    };
  }

  function html({
    props,
    React = getReact()
  }) {
    let createElement, renderToString;
    ({
      createElement,
      renderToString
    } = React);
    return "<!DOCTYPE html>\n" + renderToString(factory(createElement)(props));
  }

  function toposort(nodes) {
    let deps, start, cut;
    deps = {};
    Object.values(nodes).forEach(function (parents) {
      return parents.forEach(function (id) {
        return deps[id] = (deps[id] || 0) + 1;
      });
    });
    start = Object.keys(nodes).filter(function (it) {
      return !(it in deps);
    });

    cut = function (it) {
      let head;
      head = nodes[it].map(function (id) {
        deps[id] -= 1;
        return deps[id] === 0 ? cut(id) : [];
      });
      return [].concat(it, ...head);
    };

    return [].concat(...start.map(cut)).reverse();
  }

  function webpackAsset(source) {
    return {
      source: function () {
        return source;
      },
      size: function () {
        return source.length;
      }
    };
  }

  function htmlProps({
    chunkGroups,
    assets
  }, {
    prefix = "/",
    publicPath = prefix
  }) {
    var _context;

    let nodes, groupRef, sortedGroups, fileList, merged, list;
    nodes = Object.assign(...chunkGroups.map(function (it) {
      return {
        [it.id]: it.getParents().map(it => it.id)
      };
    }));
    groupRef = Object.assign(...chunkGroups.map(function (it) {
      return {
        [it.id]: it
      };
    }));
    sortedGroups = toposort(nodes).map(it => groupRef[it]);
    fileList = [].concat(...sortedGroups.map(function (it) {
      return [].concat(...it.chunks.map(it => it.files));
    }));
    merged = Object.assign({}, assets, ...fileList.map(function (it) {
      return {
        [it]: true
      };
    }));
    list = Object.keys(merged).map(function (it) {
      return prefix + it;
    });
    return {
      favicon: publicPath + "favicon.png",
      manifest: publicPath + "manifest.json",
      styles: list.filter((_context = /.css$/).test.bind(_context)),
      scripts: list.filter((_context = /.js$/).test.bind(_context))
    };
  }

  function generate(compilation, options) {
    let props;
    props = Object.assign({}, htmlProps(compilation, options), options);
    return {
      "index.html": webpackAsset(html({
        props,
        React: options.React
      })),
      "manifest.json": webpackAsset(JSON.stringify(manifest(options), void 8, 2))
    };
  }

  function GenerateWebApp(options) {
    this.options = options;
  }

  GenerateWebApp.prototype.apply = function (compiler) {
    return compiler.hooks.emit.tap("generate-web-app", compilation => {
      let assets;
      ({
        assets
      } = compilation);
      Object.assign(assets, generate(compilation, this.options));
    });
  };

  exports.manifest = manifest;
  exports.html = html;
  exports.GenerateWebApp = GenerateWebApp;

  return exports;

}({}));
//# sourceMappingURL=index.js.map
